import React, { Component, ReactElement, useEffect, useState } from "react";

import type { LTWH, Offset } from "../types";

interface Props {
  children: JSX.Element | null;
  style: { top: number; left: number; bottom: number };
  scrollLeft: number;
  scrollTop: number;
  pageBoundingRect: LTWH;
  containerOffset?: Offset;
}

const TipContainer = ({ children, style, scrollLeft, scrollTop, pageBoundingRect, containerOffset }: Props) => {
  const [height, setHeight] = useState(0);
  const [width, setWidth] = useState(0);

  const containerRef = React.useRef<HTMLDivElement>(null);

  const updatePosition = () => {
    const container = containerRef.current;
    if (!container) {
      return;
    }

    const { offsetHeight, offsetWidth } = container;
    setHeight(offsetHeight);
    setWidth(offsetWidth);
  };

  useEffect(() => {
    updatePosition();
  }, [children]);

  useEffect(() => {
    setTimeout(updatePosition, 0);
  }, []);

  const isStyleCalculationInProgress = width === 0 && height === 0;
  const shouldMove = style.top - height - 5 < scrollTop;
  const top = shouldMove ? style.bottom + 5 : style.top - height - 5;
  const left = Math.min(
    Math.max(style.left - width / 2, 0),
    scrollLeft + (containerOffset?.offsetWidth || 0) - width - 20
  );

  const childrenWithProps = React.Children.map(children, child =>
    // @ts-ignore
    React.cloneElement(child, {
      onUpdate: () => {
        setWidth(0);
        setHeight(0);
        setTimeout(updatePosition, 0);
      },
      popup: {
        position: shouldMove ? "below" : "above"
      }
    })
  );

  return (
    <div
      className="PdfHighlighter__tip-container"
      style={{
        visibility: isStyleCalculationInProgress ? "hidden" : "visible",
        top,
        left
      }}
      ref={containerRef}
    >
      {childrenWithProps}
    </div>
  );
};

export default TipContainer;
